<?php

namespace Drupal\document_ocr\Entity\Form\Task;

use Drupal\Core\Form\FormStateInterface;
use Drupal\document_ocr\DocumentOcrInterface;

/**
 * Process pending documents form.
 */
class ProcessForm extends BaseForm {

  protected $form_id = 'document_ocr_process_pending_form';
  protected $question = 'Process Pending Tasks';
  protected $confirm_text = 'Process Tasks';

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = parent::buildForm($form, $form_state);
    $form['#attached']['library'][] = 'document_ocr/document_ocr.admin';
    $form['description']['#prefix'] = '<p>';
    $form['description']['#suffx'] = '</p>';
    $form['actions']['cancel']['#attributes']['class'][] = 'dialog-cancel';
    $form['description']['#markup'] = $this->t('This action will lookup unprocessed files and import into destination entities. Depending on file sizes this may take time please keep the window open.');
    $form['mapping'] = [
      '#type' => 'select',
      '#title' => $this->t('Mapping to Process'),
      '#options' => $this->getPendingMappingList(),
      '#empty_option' => $this->t('- Select Mapping -'),
      '#description' => $this->t('If unselected all pending documents will be processed.'),
    ];
    $form['actions']['submit']['#prefix'] = '<div class="document-ocr-button-wrapper"><span class="document-ocr-button-spinner"></span>';
    $form['actions']['submit']['#suffix'] = '</div>';
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $batch = [
      'title' => $this->t('Processing Pending Tasks'),
      'finished' => '\Drupal\document_ocr\Services\Batch::completeProcessCallback',
      'operations' => [],
    ];
    if ($mapping = $this->entityTypeManager->getStorage('document_ocr_mapping')->load($form_state->getValue('mapping'))) {
      $batch['title'] = $this->t('Processing Pending Taks for %mapping', ['%mapping' => $mapping->label()]);
    }
    $storage = $this->entityTypeManager->getStorage('document_ocr_task');
    $query = $storage->getQuery();
    if (!empty($form_state->getValue('mapping'))) {
      $query->condition('document_ocr_mapping', $form_state->getValue('mapping'));
    }
    $query->condition('status', [DocumentOcrInterface::TASK_STATUS_INIT, DocumentOcrInterface::TASK_STATUS_PENDING], 'IN');
    $ids = $query->accessCheck(FALSE)->execute();
    $tasks = $storage->loadMultiple($ids);
    foreach ($tasks as $task) {
      $item = new \stdClass();
      $item->id = $task->id();
      $batch['operations'][] = [
        '\Drupal\document_ocr\Services\Batch::process', [$item],
      ];
    }
    batch_set($batch);
    $form_state->setRedirect('entity.document_ocr_task.collection');
  }

}
